@@ -18,19 +18,26 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
+from __future__ import print_function
+from future import standard_library
+standard_library.install_aliases()
+from builtins import input
+from builtins import str
 import os.path
 import string
-import urllib
+import urllib.request  # pylint: disable=import-error
+import urllib.parse  # pylint: disable=import-error
+import urllib.error  # pylint: disable=import-error
 
 import duplicity.backend
 from duplicity.errors import BackendException
 
 
 class GDocsBackend(duplicity.backend.Backend):
-    """Connect to remote store using Google Google Documents List API"""
+    u"""Connect to remote store using Google Google Documents List API"""
 
-    ROOT_FOLDER_ID = 'folder%3Aroot'
-    BACKUP_DOCUMENT_TYPE = 'application/binary'
+    ROOT_FOLDER_ID = u'folder%3Aroot'
+    BACKUP_DOCUMENT_TYPE = u'application/binary'
 
     def __init__(self, parsed_url):
         duplicity.backend.Backend.__init__(self, parsed_url)
@@ -43,36 +50,37 @@ class GDocsBackend(duplicity.backend.Backend):
             import gdata.client
             import gdata.docs.client
             import gdata.docs.data
-        except ImportError:
-            raise BackendException('Google Docs backend requires Google Data APIs Python '
-                                   'Client Library (see http://code.google.com/p/gdata-python-client/).')
+        except ImportError as e:
+            raise BackendException(u"""\
+Google Docs backend requires Google Data APIs Python Client Library (see http://code.google.com/p/gdata-python-client/).
+Exception: %s""" % str(e))
 
         # Setup client instance.
-        self.client = gdata.docs.client.DocsClient(source='duplicity $version')
+        self.client = gdata.docs.client.DocsClient(source=u'duplicity $version')
         self.client.ssl = True
         self.client.http_client.debug = False
-        self._authorize(parsed_url.username + '@' + parsed_url.hostname, self.get_password())
+        self._authorize(parsed_url.username + u'@' + parsed_url.hostname, self.get_password())
 
         # Fetch destination folder entry (and crete hierarchy if required).
-        folder_names = string.split(parsed_url.path[1:], '/')
+        folder_names = string.split(parsed_url.path[1:], u'/')
         parent_folder = None
         parent_folder_id = GDocsBackend.ROOT_FOLDER_ID
         for folder_name in folder_names:
-            entries = self._fetch_entries(parent_folder_id, 'folder', folder_name)
+            entries = self._fetch_entries(parent_folder_id, u'folder', folder_name)
             if entries is not None:
                 if len(entries) == 1:
                     parent_folder = entries[0]
                 elif len(entries) == 0:
-                    folder = gdata.docs.data.Resource(type='folder', title=folder_name)
+                    folder = gdata.docs.data.Resource(type=u'folder', title=folder_name)
                     parent_folder = self.client.create_resource(folder, collection=parent_folder)
                 else:
                     parent_folder = None
                 if parent_folder:
                     parent_folder_id = parent_folder.resource_id.text
                 else:
-                    raise BackendException("Error while creating destination folder '%s'." % folder_name)
+                    raise BackendException(u"Error while creating destination folder '%s'." % folder_name)
             else:
-                raise BackendException("Error while fetching destination folder '%s'." % folder_name)
+                raise BackendException(u"Error while fetching destination folder '%s'." % folder_name)
         self.folder = parent_folder
 
     def _put(self, source_path, remote_filename):
@@ -91,13 +99,13 @@ class GDocsBackend(duplicity.backend.Backend):
         if uploader:
             # Chunked upload.
             entry = gdata.docs.data.Resource(title=atom.data.Title(text=remote_filename))
-            uri = self.folder.get_resumable_create_media_link().href + '?convert=false'
+            uri = self.folder.get_resumable_create_media_link().href + u'?convert=false'
             entry = uploader.UploadFile(uri, entry=entry)
             if not entry:
-                raise BackendException("Failed to upload file '%s' to remote folder '%s'"
+                raise BackendException(u"Failed to upload file '%s' to remote folder '%s'"
                                        % (source_path.get_filename(), self.folder.title.text))
         else:
-            raise BackendException("Failed to initialize upload of file '%s' to remote folder '%s'"
+            raise BackendException(u"Failed to initialize upload of file '%s' to remote folder '%s'"
                                    % (source_path.get_filename(), self.folder.title.text))
         assert not file.close()
 
@@ -109,7 +117,7 @@ class GDocsBackend(duplicity.backend.Backend):
             entry = entries[0]
             self.client.DownloadResource(entry, local_path.name)
         else:
-            raise BackendException("Failed to find file '%s' in remote folder '%s'"
+            raise BackendException(u"Failed to find file '%s' in remote folder '%s'"
                                    % (remote_filename, self.folder.title.text))
 
     def _list(self):
@@ -122,41 +130,41 @@ class GDocsBackend(duplicity.backend.Backend):
                                       GDocsBackend.BACKUP_DOCUMENT_TYPE,
                                       filename)
         for entry in entries:
-            self.client.delete(entry.get_edit_link().href + '?delete=true', force=True)
+            self.client.delete(entry.get_edit_link().href + u'?delete=true', force=True)
 
     def _authorize(self, email, password, captcha_token=None, captcha_response=None):
         try:
             self.client.client_login(email,
                                      password,
-                                     source='duplicity $version',
-                                     service='writely',
+                                     source=u'duplicity $version',
+                                     service=u'writely',
                                      captcha_token=captcha_token,
                                      captcha_response=captcha_response)
         except gdata.client.CaptchaChallenge as challenge:
-            print('A captcha challenge in required. Please visit ' + challenge.captcha_url)
+            print(u'A captcha challenge in required. Please visit ' + challenge.captcha_url)
             answer = None
             while not answer:
-                answer = raw_input('Answer to the challenge? ')
+                answer = eval(input(u'Answer to the challenge? '))
             self._authorize(email, password, challenge.captcha_token, answer)
         except gdata.client.BadAuthentication:
             raise BackendException(
-                'Invalid user credentials given. Be aware that accounts '
-                'that use 2-step verification require creating an application specific '
-                'access code for using this Duplicity backend. Follow the instruction in '
-                'http://www.google.com/support/accounts/bin/static.py?page=guide.cs&guide=1056283&topic=1056286 '
-                'and create your application-specific password to run duplicity backups.')
+                u'Invalid user credentials given. Be aware that accounts '
+                u'that use 2-step verification require creating an application specific '
+                u'access code for using this Duplicity backend. Follow the instruction in '
+                u'http://www.google.com/support/accounts/bin/static.py?page=guide.cs&guide=1056283&topic=1056286 '
+                u'and create your application-specific password to run duplicity backups.')
 
     def _fetch_entries(self, folder_id, type, title=None):
         # Build URI.
-        uri = '/feeds/default/private/full/%s/contents' % folder_id
-        if type == 'folder':
-            uri += '/-/folder?showfolders=true'
+        uri = u'/feeds/default/private/full/%s/contents' % folder_id
+        if type == u'folder':
+            uri += u'/-/folder?showfolders=true'
         elif type == GDocsBackend.BACKUP_DOCUMENT_TYPE:
-            uri += '?showfolders=false'
+            uri += u'?showfolders=false'
         else:
-            uri += '?showfolders=true'
+            uri += u'?showfolders=true'
         if title:
-            uri += '&title=' + urllib.quote(title) + '&title-exact=true'
+            uri += u'&title=' + urllib.parse.quote(title) + u'&title-exact=true'
 
         # Fetch entries.
         entries = self.client.get_all_resources(uri=uri)
@@ -168,8 +176,8 @@ class GDocsBackend(duplicity.backend.Backend):
             for entry in entries:
                 resource_type = entry.get_resource_type()
                 if (not type) \
-                   or (type == 'folder' and resource_type == 'folder') \
-                   or (type == GDocsBackend.BACKUP_DOCUMENT_TYPE and resource_type != 'folder'):
+                   or (type == u'folder' and resource_type == u'folder') \
+                   or (type == GDocsBackend.BACKUP_DOCUMENT_TYPE and resource_type != u'folder'):
 
                     if folder_id != GDocsBackend.ROOT_FOLDER_ID:
                         for link in entry.in_collections():
@@ -185,6 +193,7 @@ class GDocsBackend(duplicity.backend.Backend):
         # Done!
         return result
 
-""" gdata is an alternate way to access gdocs, currently 05/2015 lacking OAuth support """
-duplicity.backend.register_backend('gdata+gdocs', GDocsBackend)
-duplicity.backend.uses_netloc.extend(['gdata+gdocs'])
+
+u""" gdata is an alternate way to access gdocs, currently 05/2015 lacking OAuth support """
+duplicity.backend.register_backend(u'gdata+gdocs', GDocsBackend)
+duplicity.backend.uses_netloc.extend([u'gdata+gdocs'])
